@@ -26,7 +26,9 @@ char *f_char;
 char *F_char;
 int F_flag;
 char *g_char;
+#ifdef WITH_ENGINES
 int g_flag;
+#endif
 int h_flag;
 int H_flag;
 char *l_char;
@@ -65,10 +67,11 @@ char *url_char;
 int v_flag;
 int w_flag;
 char *w_char;
+int W_flag;
 
-EVP_MD *fp_alg;
-EVP_MD *sig_alg;
-EVP_CIPHER *enc_alg;
+const EVP_MD *fp_alg;
+const EVP_MD *sig_alg;
+const EVP_CIPHER *enc_alg;
 
 static SCEP_CAP scep_caps[SCEP_CAPS] = {
 	{ .cap = SCEP_CAP_AES,      .str = "AES" }, /* AES128-CBC */
@@ -107,6 +110,67 @@ static SCEP_CAP scep_caps[SCEP_CAPS] = {
 #define SUP_CAP_STA(cap) \
 	(cap & SCEP_CAP_STA)
 
+const EVP_CIPHER *get_cipher_alg(const char *arg, int ca_caps)
+{
+	if (!arg) {
+		if (SUP_CAP_AES(ca_caps))
+			return EVP_aes_128_cbc();
+		else if (SUP_CAP_3DES(ca_caps))
+			return EVP_des_ede3_cbc();
+		else
+			return EVP_des_cbc();
+	} else if (!strncmp(arg, "blowfish", 8)) {
+		return EVP_bf_cbc();
+	} else if (!strncmp(arg, "des", 3)) {
+		return EVP_des_cbc();
+	} else if (!strncmp(arg, "3des", 4)) {
+		return EVP_des_ede3_cbc();
+	} else if (!strncmp(arg, "aes128", 6)) {
+		return EVP_aes_128_cbc();
+	} else if (!strncmp(arg, "aes192", 6)) {
+		return EVP_aes_192_cbc();
+	} else if (!strncmp(arg, "aes256", 6)) {
+		return EVP_aes_256_cbc();
+	} else if (!strncmp(arg, "aes", 3)) {
+		/* per RFC8894 "AES" represents "AES128-CBC" */
+		return EVP_aes_128_cbc();
+	} else {
+		return NULL;
+	}
+}
+
+const EVP_MD *get_digest_alg(const char *arg, int ca_caps)
+{
+	if (!arg) {
+		if (SUP_CAP_SHA_512(ca_caps))
+			return EVP_sha512();
+		else if (SUP_CAP_SHA_384(ca_caps))
+			return EVP_sha384();
+		else if (SUP_CAP_SHA_256(ca_caps))
+			return EVP_sha256();
+		else if (SUP_CAP_SHA_224(ca_caps))
+			return EVP_sha224();
+		else if (SUP_CAP_SHA_1(ca_caps))
+			return EVP_sha1();
+		else
+			return EVP_md5();
+	} else if (!strncmp(arg, "md5", 3)) {
+		return EVP_md5();
+	} else if (!strncmp(arg, "sha1", 4)) {
+		return EVP_sha1();
+	} else if (!strncmp(arg, "sha224", 6)) {
+		return EVP_sha224();
+	} else if (!strncmp(arg, "sha256", 6)) {
+		return EVP_sha256();
+	} else if (!strncmp(arg, "sha384", 6)) {
+		return EVP_sha384();
+	} else if (!strncmp(arg, "sha512", 6)) {
+		return EVP_sha512();
+	} else {
+		return NULL;
+	}
+}
+
 static char *
 handle_serial (char * serial)
 {
@@ -243,7 +307,7 @@ main(int argc, char **argv) {
 	}
 	/* Skip first parameter and parse the rest of the command */
 	optind++;
-	while ((c = getopt(argc, argv, "c:C:de:E:f:g:hF:i:k:K:l:L:n:O:p:r:Rs:S:t:T:u:vw:m:HM:")) != -1)
+	while ((c = getopt(argc, argv, "c:C:de:E:f:g:hF:i:k:K:l:L:n:O:p:r:Rs:S:t:T:u:vw:W:m:HM:")) != -1)
                 switch(c) {
 			case 'c':
 				c_flag = 1;
@@ -272,10 +336,12 @@ main(int argc, char **argv) {
 				f_flag = 1;
 				f_char = optarg;
 				break;
+#ifdef WITH_ENGINES
 			case 'g':
 				g_flag = 1;
 				g_char = optarg;
 				break;
+#endif
 			case 'h'://TODO change to eg. ID --inform=ID
 				h_flag = 1;
 				break;
@@ -326,7 +392,7 @@ main(int argc, char **argv) {
 					if(!M_char)
 						error_memory();
 					strcat(M_char, "&");
-					strncat(M_char, optarg, strlen(optarg));
+					strcat(M_char, optarg);
 				}
 				break;
 			case 'n':
@@ -376,6 +442,9 @@ main(int argc, char **argv) {
 				w_flag = 1;
 				w_char = optarg;
 				break;
+			case 'W':
+				W_flag = atoi(optarg);
+				break;
 			default:
 			  printf("argv: %s\n", argv[optind]);
 				usage();
@@ -405,11 +474,12 @@ main(int argc, char **argv) {
 		fprintf(stdout, "%s: new transaction\n", pname);
 	new_transaction(&scep_t, operation_flag);
 
+#ifdef WITH_ENGINES
 	/*enable Engine Support */
 	if (g_flag) {
 		scep_t.e = scep_engine_init();
 	}
-	
+#endif
 	/*
 	 * Check argument logic.
 	 */
@@ -568,69 +638,6 @@ main(int argc, char **argv) {
 		fprintf(stdout, "%s: port: %d\n", pname, host_port);
 	}
 
-	/* Check algorithms */
-	if (!E_flag) {
-		enc_alg = (EVP_CIPHER *)EVP_des_cbc();
-	} else if (!strncmp(E_char, "blowfish", 8)) {
-		enc_alg = (EVP_CIPHER *)EVP_bf_cbc();
-	} else if (!strncmp(E_char, "des", 3)) {
-		enc_alg = (EVP_CIPHER *)EVP_des_cbc();
-	} else if (!strncmp(E_char, "3des", 4)) {
-		enc_alg = (EVP_CIPHER *)EVP_des_ede3_cbc();
-	} else if (!strncmp(E_char, "aes128", 6)) {
-		enc_alg = (EVP_CIPHER *)EVP_aes_128_cbc();
-	} else if (!strncmp(E_char, "aes192", 6)) {
-		enc_alg = (EVP_CIPHER *)EVP_aes_192_cbc();
-	} else if (!strncmp(E_char, "aes256", 6)) {
-		enc_alg = (EVP_CIPHER *)EVP_aes_256_cbc();
-	} else if (!strncmp(E_char, "aes", 3)) {
-		/* per RFC8894 "AES" represents "AES128-CBC" */
-		enc_alg = (EVP_CIPHER *)EVP_aes_128_cbc();
-	} else {
-		fprintf(stderr, "%s: unsupported algorithm: %s\n",
-			pname, E_char);
-		exit (SCEP_PKISTATUS_ERROR);
-	}
-	if (!S_flag) {
-		sig_alg = (EVP_MD *)EVP_md5();
-	} else if (!strncmp(S_char, "md5", 3)) {
-		sig_alg = (EVP_MD *)EVP_md5();
-	} else if (!strncmp(S_char, "sha1", 4)) {
-		sig_alg = (EVP_MD *)EVP_sha1();
-	} else if (!strncmp(S_char, "sha224", 6)) {
-		sig_alg = (EVP_MD *)EVP_sha224();
-	} else if (!strncmp(S_char, "sha256", 6)) {
-		sig_alg = (EVP_MD *)EVP_sha256();
-	} else if (!strncmp(S_char, "sha384", 6)) {
-		sig_alg = (EVP_MD *)EVP_sha384();
-	} else if (!strncmp(S_char, "sha512", 6)) {
-		sig_alg = (EVP_MD *)EVP_sha512();
-	} else {
-		fprintf(stderr, "%s: unsupported algorithm: %s\n",
-			pname, S_char);
-		exit (SCEP_PKISTATUS_ERROR);
-	}
-	/* Fingerprint algorithm */
-	if (!F_flag) {
-		fp_alg = (EVP_MD *)EVP_md5();
-	} else if (!strncmp(F_char, "md5", 3)) {
-		fp_alg = (EVP_MD *)EVP_md5();
-	} else if (!strncmp(F_char, "sha1", 4)) {
-		fp_alg = (EVP_MD *)EVP_sha1();
-	} else if (!strncmp(F_char, "sha224", 6)) {
-		fp_alg = (EVP_MD *)EVP_sha224();
-	} else if (!strncmp(F_char, "sha256", 6)) {
-		fp_alg = (EVP_MD *)EVP_sha256();
-	} else if (!strncmp(F_char, "sha384", 6)) {
-		fp_alg = (EVP_MD *)EVP_sha384();
-	} else if (!strncmp(F_char, "sha512", 6)) {
-		fp_alg = (EVP_MD *)EVP_sha512();
-	} else {
-		fprintf(stderr, "%s: unsupported algorithm: %s\n",
-			pname, F_char);
-		exit (SCEP_PKISTATUS_ERROR);
-	}
-
 	if (v_flag)
 		fprintf(stdout, "%s: SCEP_OPERATION_GETCAPS\n",
 			pname);
@@ -644,6 +651,9 @@ main(int argc, char **argv) {
 		exit (SCEP_PKISTATUS_NET);
 	}
 
+	if (v_flag)
+		fprintf(stdout, "%s\n", reply.payload);
+
 	if (reply.status == 200 && reply.payload != NULL) {
 		for ( i = 0 ; i < reply.bytes ; ) {
 			int _ca_caps = 0;
@@ -688,6 +698,24 @@ main(int argc, char **argv) {
 					pname, ca_caps);
 	}
 
+	/* Check algorithms */
+	if ((enc_alg = get_cipher_alg(E_char, ca_caps)) == NULL) {
+		fprintf(stderr, "%s: unsupported algorithm: %s\n",
+			pname, E_char);
+		exit (SCEP_PKISTATUS_ERROR);
+	}
+	if ((sig_alg = get_digest_alg(S_char, ca_caps)) == NULL) {
+		fprintf(stderr, "%s: unsupported algorithm: %s\n",
+			pname, S_char);
+		exit (SCEP_PKISTATUS_ERROR);
+	}
+	/* Fingerprint algorithm */
+	if ((fp_alg = get_digest_alg(F_char, ca_caps)) == NULL) {
+		fprintf(stderr, "%s: unsupported algorithm: %s\n",
+			pname, F_char);
+		exit (SCEP_PKISTATUS_ERROR);
+	}
+
 	/*
 	 * Switch to operation specific code
 	 */
@@ -922,11 +950,12 @@ main(int argc, char **argv) {
 			  exit (SCEP_PKISTATUS_FILE);
 			}
 			
-			if(g_flag) {
+#ifdef WITH_ENGINES
+			if(g_flag)
 				sscep_engine_read_key_new(&rsa, k_char, scep_t.e);
-			} else {
+			else
+#endif
 				rsa = read_key(k_char);
-			}
 
 
 			if ((K_flag && !O_flag) || (!K_flag && O_flag)) {
@@ -936,11 +965,12 @@ main(int argc, char **argv) {
 
 			if (K_flag) {
 				//TODO auf hwcrhk prfen?
-				if(g_flag) {
+#ifdef WITH_ENGINES
+				if(g_flag)
 					sscep_engine_read_key_old(&renewal_key, K_char, scep_t.e);
-				} else {
+				else
+#endif
 					renewal_key = read_key(K_char);
-				}
 			}
 
 			if (O_flag) {
@@ -1245,12 +1275,15 @@ usage() {
 	"  -u <url>          SCEP server URL\n"
 	"  -p <host:port>    Use proxy server at host:port\n"
 	"  -M <string>       Monitor Information String name=value&name=value ...\n"
+#ifdef WITH_ENGINES
 	"  -g <engine>       Use the given cryptographic engine\n"
-	"  -h				 Keyforme=ID. \n"//TODO
+#endif
+	"  -h                Keyforme=ID. \n"//TODO
 	"  -f <file>         Use configuration file\n"
 	"  -c <file>         CA certificate file or '-n' suffixed files (write if OPERATION is getca)\n"
 	"  -E <name>         PKCS#7 encryption algorithm (des|3des|blowfish|aes[128]|aes192|aes256)\n"
 	"  -S <name>         PKCS#7 signature algorithm (md5|sha1|sha224|sha256|sha384|sha512)\n"
+	"  -W <secs>         Wait for connectivity, up to <secs> seconds\n"
 	"  -v                Verbose output (for debugging the configuration)\n"
 	"  -d                Debug output (more verbose, for debugging the implementation)\n"
 	"\nOPTIONS for OPERATION getca are\n"
